@@ -9,11 +9,12 @@ actions for carburetor
 import os
 import re
 import signal
+from subprocess import PIPE, Popen
 
-from gi.repository import Gio, Adw
+from gi.repository import Adw, GLib, Gio
+from tractor import bridges, checks, db, proxy, actions as tactions
 
 from . import config
-from . import tasks
 from . import ui
 
 
@@ -51,6 +52,11 @@ def on_preferences(*argv) -> None:
         prefs_window = ui.get("PreferencesWindow")
         prefs_window.set_transient_for(app.window)
         app.prefs = prefs_window
+        action = Gio.SimpleAction.new("save")
+        action.connect("activate", on_save)
+        action_group = Gio.SimpleActionGroup()
+        action_group.add_action(action)
+        prefs_window.insert_action_group("app", action_group)
     app.prefs.show()
 
 
@@ -78,6 +84,7 @@ def on_about(*argv) -> None:
         about_window.set_developers(["Danial Behzadi <dani.behzi@ubuntu.com>"])
         about_window.set_translator_credits(config._("translator-credits"))
         about_window.set_transient_for(app.window)
+        about_window.set_hide_on_close(True)
         app.about = about_window
     app.about.show()
 
@@ -86,7 +93,7 @@ def on_quit(*argv) -> None:
     """
     exit the app
     """
-    tasks.kill_tor()
+    tactions.kill_tor()
     app = argv[2]
     app.quit()
 
@@ -99,53 +106,59 @@ def on_connect(*argv) -> None:
     button = ui.get("SplitButton")
     button.set_sensitive(False)
     progress_bar = ui.get("ProgressBar")
-    set_progress(0)
+    ui.set_progress_bar(0)
     progress_bar.show()
     cancel_button = ui.get("CancelButton")
     cancel_button.set_visible(True)
+    ui.set_orbi("load")
     page = ui.get("MainPage")
-    page.set_icon_name("image-loading-symbolic")
-    if tasks.is_running():
+    window = ui.get("MainWindow")
+    if checks.running():
         text_stopping = config._("Disconnecting…")
         page.set_title(text_stopping)
         action = "stop"
+        window.set_hide_on_close(False)
     else:
         text_starting = config._("Connecting…")
         page.set_title(text_starting)
         action = "start"
-    tasks.connect(action, app)
+        window.set_hide_on_close(True)
+    connect(action, app)
 
 
 def on_new_id(*_, **__) -> None:
     """
     clicking on new id button
     """
-    if tasks.is_running():
-        tasks.new_id()
+    if checks.running():
+        tactions.new_id()
         toast = config._("You have a new identity!")
     else:
         toast = config._("Tractor is not running!")
-    notify(toast)
+    ui.notify(toast)
 
 
-def on_check(*_, **__) -> None:
+def on_check(*argv) -> None:
     """
     checks if tractor is connected or not
     """
-    tasks.is_connected()
+    app = argv[2]
+    ui.set_orbi("load")
+    check = Popen([config.COMMAND, "isconnected"], stdout=PIPE)
+    app.io = GLib.io_add_watch(check.stdout, GLib.IO_HUP, connected_hup, app)
 
 
 def on_toggle_proxy(*_, **__) -> None:
     """
     toggle proxy mode on system
     """
-    if tasks.is_proxy_set():
-        tasks.set_proxy("unset")
+    if checks.proxy_set():
+        proxy.proxy_unset()
         toast = config._("Proxy has been unset")
     else:
-        tasks.set_proxy("set")
+        proxy.proxy_set()
         toast = config._("Proxy has been set")
-    notify(toast)
+    ui.notify(toast)
 
 
 def on_cancel(*_, **__) -> None:
@@ -165,131 +178,80 @@ def on_save(*_, **__) -> None:
     textview = ui.get("BridgesTextView")
     buff = textview.get_buffer()
     text = buff.get_text(buff.get_start_iter(), buff.get_end_iter(), 0)
-    regex = re.compile(r"^( )*([Bb][Rr][Ii][Dd][Gg][Ee])?( )*", re.MULTILINE)
-    bridges_file = tasks.get_bridges_file()
-    if text == regex.sub("Bridge ", text):
-        with open(bridges_file, "w", encoding="utf-8") as file:
-            file.write(text)
-    else:
+    pt_type = db.get_val("bridge-type")
+    try:
+        relevant = bridges.relevant_lines(text, pt_type)
+    except ValueError:
+        relevant = True
+    if not relevant:
         dialog = ui.get("BridgErrorDialog")
         dialog.show()
+    bridges_file = bridges.get_file()
+    with open(bridges_file, "w", encoding="utf-8") as file:
+        file.write(text)
 
 
-def set_description(text: str) -> None:
+def connect(action: str, app) -> None:
     """
-    set description on main page
+    connect or disconnect
     """
-    page = ui.get("MainPage")
-    page.set_description(text)
-
-
-def set_progress(percentage: int) -> None:
-    """
-    set progressbar percentage
-    """
-    progress_bar = ui.get("ProgressBar")
-    fraction = float(percentage) / 100
-    progress_bar.set_fraction(fraction)
-
-
-def notify(text: str) -> None:
-    """
-    show toast
-    """
-    overlay = ui.get("ToastOverlay")
-    toast = Adw.Toast()
-    toast.set_title(text)
-    overlay.add_toast(toast)
-
-
-def set_to_stopped(app) -> None:
-    """
-    set status to stopped
-    """
-    page = ui.get("MainPage")
-    page.set_icon_name("process-stop-symbolic")
-    page.set_title(config._("Stopped"))
-    page.set_description("")
-    button = ui.get("SplitButton")
-    text_start = config._("_Connect")
-    style = button.get_style_context()
-    style.remove_class("destructive-action")
-    style.add_class("suggested-action")
-    button.set_label(text_start)
-    action_menu = button.get_popover()
-    action_menu.set_sensitive(False)
-    button = ui.get("CancelButton")
-    button.set_visible(False)
-    dconf = config.dconf
-    dconf.reset("pid")
-    if app:  # don't run on startup
-        notify(config._("Tractor is stopped"))
-
-
-def set_to_running(app) -> None:
-    """
-    set status to connected
-    """
-    page = ui.get("MainPage")
-    page.set_icon_name("security-high-symbolic")
-    page.set_title(config._("Running"))
-    page.set_description(
-        f"{config._('Socks Port')}: {config.dconf.get_int('socks-port')}\n"
-        f"{config._('DNS Port')}: {config.dconf.get_int('dns-port')}\n"
-        f"{config._('HTTP Port')}: {config.dconf.get_int('http-port')}"
+    task = Popen(
+        [config.COMMAND, action, "--verbose"],
+        stdout=PIPE,
+        start_new_session=True,
     )
-    button = ui.get("SplitButton")
-    text_stop = config._("_Disconnect")
-    style = button.get_style_context()
-    style.remove_class("suggested-action")
-    style.add_class("destructive-action")
-    button.set_label(text_stop)
-    action_menu = button.get_popover()
-    action_menu.set_sensitive(True)
-    button = ui.get("CancelButton")
-    button.set_visible(False)
-    if app:  # don't run on startup
-        notify(config._("Tractor is running"))
+    if action == "start":
+        config.dconf.set_int("pid", task.pid)
+    elif checks.proxy_set():
+        proxy.proxy_unset()
+    app.io_in = GLib.io_add_watch(task.stdout, GLib.IO_IN, set_progress)
+    GLib.io_add_watch(task.stdout, GLib.IO_HUP, thread_finished, app)
 
 
-def set_run_status(app=None) -> None:
+def set_progress(stdout, *_) -> bool:
     """
-    set status of conection
+    set progress output on UI
     """
-    if tasks.is_running():
-        set_to_running(app)
-    else:
-        set_to_stopped(app)
-    button = ui.get("SplitButton")
-    button.set_sensitive(True)
-    progress_bar = ui.get("ProgressBar")
-    progress_bar.hide()
+    try:
+        line = stdout.readline().decode("utf-8")
+        ui.add_to_terminal(line[5:-5])
+        if "Bootstrapped" in line:
+            valid = re.compile(r".*Bootstrapped .+% \(.*\): ")
+            notice = valid.sub("", line)[:-5]
+            ui.set_description(notice)
+            percentage = line.split(" ")[5]
+            ui.set_progress_bar(int(percentage[:-1]))
+    except ValueError:
+        return False
+    return True
 
 
-def set_pluginrow_sensivity(row=None) -> None:
+def thread_finished(stdout, condition, app) -> bool:
     """
-    set row sensitive if a plugable transport is set
+    things to do after process finished
     """
-    if not row:
-        row = ui.get("PluginRow")
-    bridgetype = config.dconf.get_int("bridge-type")
-    if bridgetype > 1:
-        row.set_sensitive(True)
-    else:
-        row.set_sensitive(False)
+    if condition:
+        GLib.source_remove(app.io_in)
+        stdout.close()
+        ui.set_run_status(app)
+        return False
+    return True
 
 
-def setup_pluginbutton(button=None) -> None:
+def connected_hup(stdout, condition, app) -> bool:
     """
-    set plugin button label and chooser
+    return connection status
     """
-    if not button:
-        button = ui.get("PluginButton")
-    dconf = config.dconf
-    filename = Gio.File.new_for_path(dconf.get_string("plugable-transport"))
-    if filename.query_exists():
-        basename = filename.get_basename()
-        button.set_label(basename)
-    chooser = ui.get("PluginChooser")
-    chooser.set_initial_file(filename)
-    button.chooser = chooser
+    if condition:
+        result = stdout.readline().decode("utf-8")
+        if "True" in result:
+            toast = config._("Tractor is connected")
+            ui.set_orbi("run")
+        else:
+            toast = config._("Tractor couldn't connect")
+            ui.set_orbi("dead")
+        ui.notify(toast)
+        GLib.source_remove(app.io)
+        stdout.close()
+        return True
+    return False
