@@ -1,6 +1,6 @@
-#!/usr/bin/python3
+# Part of Carburetor project
 # Released under GPLv3+ License
-# Danial Behzadi<dani.behzi@ubuntu.com>, 2019-2022.
+# Danial Behzadi<dani.behzi@ubuntu.com>, 2019-2023.
 
 """
 Main module for carburetor
@@ -10,7 +10,7 @@ from sys import argv
 
 import gi
 
-gi.require_versions({"Adw": "1"})
+gi.require_versions({"Adw": "1", "Gtk": "4.0"})
 from gi.repository import Adw
 
 from . import actions
@@ -29,6 +29,7 @@ class Application(Adw.Application):
         self.window = None
         self.prefs = None
         self.about = None
+        self.shortcuts = None
 
     def do_startup(self, *args, **kwargs) -> None:
         Adw.Application.do_startup(self)
@@ -49,7 +50,10 @@ def main() -> None:
     """
     ui.initialize_builder()
     app = Application()
-    app.run(argv)
+    try:
+        app.run(argv)
+    except KeyboardInterrupt:
+        app.quit()
 
 
 if __name__ == "__main__":
