@@ -1,6 +1,6 @@
-#!/usr/bin/python3
+# Part of Carburetor project
 # Released under GPLv3+ License
-# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2022.
+# Danial Behzadi<dani.behzi@ubuntu.com>, 2020-2023.
 
 
 """
@@ -13,9 +13,6 @@ import locale
 import os
 import sys
 
-import gi
-
-gi.require_versions({"Adw": "1"})
 from gi.repository import Gio, GLib, Adw
 
 
